import os
import shutil
import ujson

from edge import eventlog
from edge.path_helpers import *
from os.path import join as join_path

import edge.pipeline_helpers as helpers


# def __enter__(self):
#     eventlog.log_d("try")
#     return self
#
# def __exit__(self, x_type, x_value, x_traceback):
#     if x_type is None:
#         eventlog.log_d("done")


class OldFileBackup:
    def __init__(self, source_filepath, backup_filepath):
        """
        :type backup_filepath: str
        :type source_filepath: str
        """
        self._backup_filepath = backup_filepath
        self._source_filepath = source_filepath

    @staticmethod
    def try_create(backup_dir_path, ui_pid, ui_identity):
        """
        :type backup_dir_path: str
        :type ui_pid: str
        :type ui_identity: UiIdentity
        :rtype: OldFileBackup | None
        """
        # backup configuration files
        source_filepath = get_ui_pipeline_jnode_filepath(ui_pid, ui_identity.type, ui_identity.id)
        backup_name = 'ui-%s-%s-old.json' % (ui_identity.type, ui_pid)
        backup_filepath = join_path(backup_dir_path, backup_name)
        try:
            eventlog.log_d("try: backup %s to %s" % (source_filepath, backup_filepath))
            shutil.copy2(source_filepath, backup_filepath)
        except (IOError, os.error, shutil.Error) as x:
            eventlog.log_d("fail: cp %s to %s (%s)" % (source_filepath, backup_filepath, x.message))
            return None

        # handle backup succcess
        jnode_backup = OldFileBackup(source_filepath, backup_filepath)
        eventlog.log_d("done")
        return jnode_backup

    def try_restore(self):
        """
         :rtype: bool
         """
        try:
            eventlog.log_d("try: restore %s to %s" % (self._backup_filepath, self._source_filepath))
            shutil.copy2(self._backup_filepath, self._source_filepath)
            eventlog.log_d("done")
            return True

        except (os.error, shutil.Error) as x:
            eventlog.log_d("fail: cp %s to %s (%s)" % (self._backup_filepath, self._source_filepath, x.message))

        return False

    def remove(self):
        eventlog.log_d('delete %s' % self._backup_filepath)
        os.remove(self._backup_filepath)


class OldDirBackup:
    def __init__(self, source_dirpath, backup_dirpath):
        """
        :type source_dirpath: str
        :type backup_dirpath: str
        """
        self._source_dirpath = source_dirpath
        self._backup_dirpath = backup_dirpath

    @staticmethod
    def try_create(dir_path, ui_pid, edge_identity):
        """
        :type dir_path: str
        :type ui_pid: str
        :type edge_identity: EdgeIdentity
        :rtype: OldDirBackup | None
        """
        # create backup directory.
        edge_group = helpers.get_edge_group(ui_pid)
        edge_id = helpers.get_edge_id(edge_group, edge_identity)
        backup_dirpath = join_path(dir_path, 'edge-%s.%s' % (edge_identity.type, edge_id))

        # backup configuration dirs
        source_dirpath = join_path(BASE_EDGE_JNODE_DIRPATH, '%s.%s' % (edge_identity.type, edge_id))

        try:
            eventlog.log_d("try: backup %s to %s" % (source_dirpath, backup_dirpath))
            shutil.copytree(source_dirpath, backup_dirpath)
        except (IOError, os.error, shutil.Error) as x:
            eventlog.log_d("fail: backup %s to %s (%s)" % (source_dirpath, backup_dirpath, x.message))
            return None

        dir_backup = OldDirBackup(source_dirpath, backup_dirpath)
        eventlog.log_d("done")
        return dir_backup

    def try_restore(self):
        """
         :rtype: bool
         """
        try:
            eventlog.log_d("try: restore %s to %s" % (self._backup_dirpath, self._source_dirpath))
            shutil.rmtree(self._source_dirpath, ignore_errors=False,
                          onerror=lambda func, path, exc_info: eventlog.log_d(
                              "fail: rm %s (%s)" % (path, exc_info[1].message)))

            shutil.copytree(self._backup_dirpath, self._source_dirpath)

            eventlog.log_d("done")
            return True

        except shutil.Error as x:
            for srcname, dstname, why in x.args[0]:
                eventlog.log_d("fail: cp %s to %s (%s)" % (srcname, dstname, why))

        return False

    def remove(self):
        eventlog.log_d('delete %s' % self._backup_dirpath)
        shutil.rmtree(self._backup_dirpath, ignore_errors=False,
                      onerror=lambda func, path, exc_info: eventlog.log_d(
                          "fail: rm %s (%s)" % (path, exc_info[1].message)))


class NewFileBackup:
    def __init__(self, backup_filepath):
        """
        :type backup_filepath: str
        """
        self._backup_filepath = backup_filepath

    @staticmethod
    def try_create(backup_dir_path, ui_pid, ui_identity, new_jnode):
        """
        :type backup_dir_path: str
        :type ui_pid: str
        :type ui_identity: UiIdentity
        :type new_jnode: dict
        :rtype: NewFileBackup | None
        """
        # backup configuration files
        new_name = 'ui-%s-%s-new.json' % (ui_identity.type, ui_pid)
        new_filepath = join_path(backup_dir_path, new_name)
        try:
            eventlog.log_d("try: create %s" % new_filepath)
            with open(new_filepath, 'w') as json_file:
                ujson.dump(new_jnode, json_file, indent=2)

        except Exception as x:
            eventlog.log_e(x.message)

        # handle backup succcess
        jnode_backup = NewFileBackup(new_filepath)
        eventlog.log_d("done")
        return jnode_backup

    def remove(self):
        os.remove(self._backup_filepath)

    @staticmethod
    def try_restore():
        """
         :rtype: bool
         """
        return True


