import ujson
import os

from edge.admin.ui_types import BaseCreator
from edge.path_helpers import *

import edge.pipeline_helpers as helpers
from os.path import join as join_path


def tr_try_get_creator_ctor(ui_entry_point, used_ui_modules, used_edge_modules, avaliable_ui_modules):
    """
    :type ui_entry_point: UiIdentity
    :type used_ui_modules: set(str, str)
    :type used_edge_modules: set(str, str)
    :type avaliable_ui_modules: set of UiIdentity
    :rtype: None | (str, UiIdentity, dict of UiIdentity: dict) -> BaseCreator
    """
    creator_ctor = None
    # speed
    if ui_entry_point in [Pipeline.Speed.Ui.HTTP_STREAMING_0, Pipeline.Speed.Ui.VIDEOSOURCE_0,
                          Pipeline.Speed.Ui.RTSP_STREAMING_0, Pipeline.Speed.Ui.TRANSIT_0]:
        used_ui_modules.update([Pipeline.Speed.Ui.HTTP_STREAMING_0, Pipeline.Speed.Ui.VIDEOSOURCE_0,
                                Pipeline.Speed.Ui.RTSP_STREAMING_0, Pipeline.Speed.Ui.TRANSIT_0])
        used_edge_modules.add(Pipeline.Speed.Edge.TRACKER_0)
        creator_ctor = TrCreator.create

    elif ui_entry_point in [Pipeline.Speed.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Speed.Ui.HTTP_STREAMING_0, Pipeline.Speed.Ui.VIDEOSOURCE_0,
                                Pipeline.Speed.Ui.RTSP_STREAMING_0, Pipeline.Speed.Ui.TRANSIT_0])
        creator_ctor = TrCreator.create

    # mv
    elif ui_entry_point in [Pipeline.Mv.Ui.HTTP_STREAMING_0, Pipeline.Mv.Ui.VIDEOSOURCE_0,
                            Pipeline.Mv.Ui.RTSP_STREAMING_0, Pipeline.Mv.Ui.TRANSIT_0]:
        used_ui_modules.update([Pipeline.Mv.Ui.HTTP_STREAMING_0, Pipeline.Mv.Ui.VIDEOSOURCE_0,
                                Pipeline.Mv.Ui.RTSP_STREAMING_0, Pipeline.Mv.Ui.TRANSIT_0])
        used_edge_modules.add(Pipeline.Mv.Edge.TRACKER_0)
        creator_ctor = TrCreator.create

    elif ui_entry_point in [Pipeline.Mv.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Mv.Ui.HTTP_STREAMING_0, Pipeline.Mv.Ui.VIDEOSOURCE_0,
                                Pipeline.Mv.Ui.RTSP_STREAMING_0, Pipeline.Mv.Ui.TRANSIT_0])
        creator_ctor = TrCreator.create

    # rtsp
    elif ui_entry_point in [Pipeline.Rtsp.Ui.HTTP_STREAMING_0, Pipeline.Rtsp.Ui.VIDEOSOURCE_0,
                            Pipeline.Rtsp.Ui.TRANSIT_0]:
        used_ui_modules.update([Pipeline.Rtsp.Ui.HTTP_STREAMING_0, Pipeline.Rtsp.Ui.VIDEOSOURCE_0,
                                Pipeline.Rtsp.Ui.TRANSIT_0])
        used_edge_modules.add(Pipeline.Rtsp.Edge.TRACKER_0)
        creator_ctor = TrCreator.create

    elif ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Rtsp.Ui.HTTP_STREAMING_0, Pipeline.Rtsp.Ui.VIDEOSOURCE_0,
                                Pipeline.Rtsp.Ui.TRANSIT_0])
        creator_ctor = TrCreator.create

    return creator_ctor


class TrCreator(BaseCreator):
    def __init__(self, id_main_misc_setupstore):
        """
        :type id_main_misc_setupstore: str, dict, dict, str
        """
        BaseCreator.__init__(self)

        self._id_main_misc_setupstore = id_main_misc_setupstore

    @staticmethod
    def create(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: TrCreator
        """
        # speed
        if ui_entry_point in [Pipeline.Speed.Ui.PIPELINE, Pipeline.Speed.Ui.VIDEOSOURCE_0,
                              Pipeline.Speed.Ui.HTTP_STREAMING_0, Pipeline.Speed.Ui.RTSP_STREAMING_0,
                              Pipeline.Speed.Ui.TRANSIT_0]:
            id_main_misc_setupstore = TrCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = TrCreator(id_main_misc_setupstore)

        # mv
        elif ui_entry_point in [Pipeline.Mv.Ui.PIPELINE, Pipeline.Mv.Ui.VIDEOSOURCE_0,
                                Pipeline.Mv.Ui.HTTP_STREAMING_0, Pipeline.Mv.Ui.RTSP_STREAMING_0,
                                Pipeline.Mv.Ui.TRANSIT_0]:
            id_main_misc_setupstore = TrCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = TrCreator(id_main_misc_setupstore)

        # rtsp
        elif ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE, Pipeline.Rtsp.Ui.VIDEOSOURCE_0,
                                Pipeline.Rtsp.Ui.HTTP_STREAMING_0, Pipeline.Rtsp.Ui.TRANSIT_0]:
            id_main_misc_setupstore = TrCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = TrCreator(id_main_misc_setupstore)

        else:
            raise NotImplemented('pipeline not supported')

        return creator

    @staticmethod
    def _serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: str, dict, dict, str
        """
        or_gid = None
        ui_rtsp_streaming_jnode = None
        ui_transit_rules_jnode = None

        if ui_entry_point.ptype == PTYPE_SPEED:
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.VIDEOSOURCE_0]
            ui_http_streaming_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.HTTP_STREAMING_0]
            ui_rtsp_streaming_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.RTSP_STREAMING_0]
            ui_transit_rules_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.TRANSIT_0]
            tr_edge_identity = Pipeline.Speed.Edge.TRACKER_0
            or_gid = Pipeline.Speed.Edge.RENDERER_1.gid

        elif ui_entry_point.ptype == PTYPE_LPR_RTSP:
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.VIDEOSOURCE_0]
            ui_http_streaming_jnode = ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.HTTP_STREAMING_0]
            ui_transit_rules_jnode = ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.TRANSIT_0]
            tr_edge_identity = Pipeline.Rtsp.Edge.TRACKER_0

        elif ui_entry_point.ptype == PTYPE_LPR_MV:
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.VIDEOSOURCE_0]
            ui_http_streaming_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.HTTP_STREAMING_0]
            ui_rtsp_streaming_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.RTSP_STREAMING_0]
            ui_transit_rules_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.TRANSIT_0]
            tr_edge_identity = Pipeline.Mv.Edge.TRACKER_0
            or_gid = Pipeline.Mv.Edge.RENDERER_1.gid

        else:
            raise NotImplemented

        spool_dir = get_spool_dir(ui_pid)
        edge_gid = tr_edge_identity.gid

        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_TRACKER, edge_gid)
        main_conf_abspath = os.path.join(edge_jnode_dir, 'main.conf')
        old_main_conf_jnode = dict()
        if os.path.exists(main_conf_abspath):
            with open(main_conf_abspath, 'r') as json_file:
                old_main_conf_jnode = ujson.load(json_file)

        if ui_entry_point.ptype in [PTYPE_SPEED, PTYPE_LPR_MV]:
            if ui_videosource_jnode['ro'].get('raw-format', 'MONO8T') == 'MONO8T':
                sourse_dir = join_path(spool_dir, 'mg%s-tr%s-scene' % (edge_gid, edge_gid))
            else:
                sourse_dir = join_path(spool_dir, 'rc%s-tr%s-scene' % (edge_gid, edge_gid))
        else:
            sourse_dir = join_path(spool_dir, 'rg%s-tr%s-scene' % (edge_gid, edge_gid))

        # create main.conf
        main_conf = {
            "systest": old_main_conf_jnode.get('systest', False),
            "plate-detected-mode": ui_transit_rules_jnode['ro'].get("plate-detected-mode", False),
            "Monitoring": {
                "StatFile": join_path(spool_dir, 'zabbix', 'tr%s' % edge_gid),
                "StatTout": 5
            },
            "FileApi": {
                "AddTrackReact": {
                    "Dir": join_path(spool_dir, 'lr%s-tr%s-track' % (edge_gid, edge_gid))
                },
                "AddSceneReact": {
                    "MgDir": sourse_dir,
                    "LrDir": join_path(spool_dir, 'lr%s-tr%s-scene' % (edge_gid, edge_gid)),
                    "Tout": 750
                },
                "TrackEvent": {
                    "endpoints": [
                        join_path(spool_dir, 'tr%s-sc%s-track' % (edge_gid, edge_gid))
                    ],
                    "debounce-tout": 3 # TODO: add tout when refactor debouncer
                },
                "SceneEvent": []
            }
        }

        if ui_entry_point.ptype in [PTYPE_LPR_RTSP, PTYPE_LPR_MV, PTYPE_SPEED]:
            main_conf['store'] = {
                "max-file-count": 20,  # max-file-count >> debounce-tout * lr_track_fps (or we'll lose some events)
                "endpoint": join_path(spool_dir, 'tr%s-store' % edge_gid)
            }

        # enable http streaming
        if ui_http_streaming_jnode['rw']['enabled']:
            # noinspection PyUnresolvedReferences
            main_conf['FileApi']['SceneEvent'].append({
                'SkipRate': ui_http_streaming_jnode['rw']['live']['videostream']['skip-rate'],
                'Dir': join_path(spool_dir, 'tr%s-or%s-scene' % (edge_gid, edge_gid))})

        # enable rtsp-streaming
        if ui_entry_point.ptype == PTYPE_LPR_MV or ui_entry_point.ptype == PTYPE_SPEED:
            if ui_rtsp_streaming_jnode['rw']['enabled']:
                # noinspection PyUnresolvedReferences
                main_conf['FileApi']['SceneEvent'].append({
                    'SkipRate': ui_rtsp_streaming_jnode['rw']['live']['videostream']['skip-rate'],
                    'Dir': join_path(spool_dir, 'tr%s-or%s-scene' % (edge_gid, or_gid))})

        # add ra-dir for speed pipelines
        if ui_entry_point.ptype == PTYPE_SPEED:
            main_conf['FileApi']['AddSceneReact']['RaDir'] = join_path(
                spool_dir, 'ra%s-tr%s-scene' % (edge_gid, edge_gid))

        # create misc.conf
        misc_jnode = {
            "min_valid_radar_track_length": 5,
            "lpr_track_caching_time": 4,
            "radar_track_caching_time": 3
        }
        # create setupstore.conf
        edge_group = helpers.get_edge_group(ui_pid)
        edge_id = helpers.get_edge_id(edge_group, tr_edge_identity)
        setupstore_conf_text = get_setupstore_conf_text(edge_id)

        return edge_id, main_conf, misc_jnode, setupstore_conf_text

    def serialize(self):
        edge_id, main_conf_jnode, misc_jnode, setupstore_conf_text = self._id_main_misc_setupstore

        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_TRACKER, edge_id)
        if not os.path.exists(edge_jnode_dir):
            helpers.silent_mkdir_r(edge_jnode_dir)

        main_conf_abspath = join_path(edge_jnode_dir, 'main.conf')
        with open(main_conf_abspath, 'w') as json_file:
            ujson.dump(main_conf_jnode, json_file, indent=2)

        setupstore_conf_filepath = join_path(edge_jnode_dir, 'setupstore.conf')
        with open(setupstore_conf_filepath, 'w') as text_file:
            text_file.write(setupstore_conf_text)

        misc_conf_filepath = join_path(edge_jnode_dir, 'misc.conf')
        with open(misc_conf_filepath, 'w') as json_file:
            ujson.dump(misc_jnode, json_file, indent=2)


class TrDeleter:
    def __init__(self, edge_tr_dirpath):
        """
        :type edge_tr_dirpath: str
        """
        self._edge_tr_dirpath = edge_tr_dirpath

    @staticmethod
    def create(ui_pid, ui_entry_point):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :rtype: TrDeleter
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point == Pipeline.Rtsp.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Rtsp.Edge.TRACKER_0)
            edge_type = Pipeline.Rtsp.Edge.TRACKER_0.type
            deleter = TrDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        if ui_entry_point == Pipeline.Mv.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.TRACKER_0)
            edge_type = Pipeline.Mv.Edge.TRACKER_0.type
            deleter = TrDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Speed.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.TRACKER_0)
            edge_type = Pipeline.Speed.Edge.TRACKER_0.type
            deleter = TrDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        else:
            raise NotImplemented

    def run(self):
        helpers.silent_rm_r(self._edge_tr_dirpath)
